"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PolygonDidRegistrar = void 0;
const polygon_did_resolver_1 = require("@ayanworks/polygon-did-resolver");
const askar_1 = require("@credo-ts/askar");
const core_1 = require("@credo-ts/core");
const did_resolver_1 = require("did-resolver");
const ethers_1 = require("ethers");
const ledger_1 = require("../ledger");
const didPolygonUtil_1 = require("./didPolygonUtil");
class PolygonDidRegistrar {
    constructor() {
        this.supportedMethods = ['polygon'];
        this.resolver = new did_resolver_1.Resolver((0, polygon_did_resolver_1.getResolver)());
    }
    async create(agentContext, options) {
        const ledgerService = agentContext.dependencyManager.resolve(ledger_1.PolygonLedgerService);
        const didRepository = agentContext.dependencyManager.resolve(core_1.DidRepository);
        const privateKey = options.secret.privateKey;
        const wallet = new ethers_1.Wallet(new ethers_1.SigningKey(privateKey));
        const provider = new ethers_1.JsonRpcProvider(ledgerService.rpcUrl);
        const value = await provider.getBalance(wallet.address);
        if (Number(value) == 0) {
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: 'Insufficient balance in wallet',
                },
            };
        }
        const key = await agentContext.wallet.createKey({ keyType: core_1.KeyType.K256, privateKey });
        const publicKeyHex = key.publicKey.toString('hex');
        const did = (0, didPolygonUtil_1.buildDid)(options.method, options.options.network, publicKeyHex);
        agentContext.config.logger.info(`Creating DID on ledger: ${did}`);
        try {
            const signingKey = await this.getSigningKey(agentContext.wallet, key.publicKeyBase58);
            const didRegistry = ledgerService.createDidRegistryInstance(signingKey);
            // Create did document
            const secpDidDoc = (0, didPolygonUtil_1.getSecp256k1DidDoc)(did, key, options.options.endpoint);
            const response = await didRegistry.create(did, secpDidDoc);
            agentContext.config.logger.info(`Published did on ledger: ${did}`);
            const didDoc = response.didDoc;
            const didDocument = core_1.JsonTransformer.fromJSON(didDoc, core_1.DidDocument);
            const didRecord = new core_1.DidRecord({
                did: didDocument.id,
                role: core_1.DidDocumentRole.Created,
                didDocument,
            });
            agentContext.config.logger.info(`Saving DID record to wallet: ${did} and did document: ${didDocument}`);
            await didRepository.save(agentContext, didRecord);
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {
                    txn: response.txnHash,
                },
                didState: {
                    state: 'finished',
                    did: didDocument.id,
                    didDocument: didDocument,
                },
            };
        }
        catch (error) {
            const errorMessage = error instanceof Error ? error.message : 'Unknown error';
            agentContext.config.logger.error(`Error registering DID ${did} : ${errorMessage}`);
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: `unknownError: ${errorMessage}`,
                },
            };
        }
    }
    async update(agentContext, options) {
        var _a, _b, _c, _d, _e;
        const ledgerService = agentContext.dependencyManager.resolve(ledger_1.PolygonLedgerService);
        const didRepository = agentContext.dependencyManager.resolve(core_1.DidRepository);
        let didDocument;
        let didRecord;
        try {
            const isValidDidDoc = (0, didPolygonUtil_1.validateSpecCompliantPayload)(options.didDocument);
            if (options.didDocument && isValidDidDoc === null) {
                didDocument = options.didDocument;
                const resolvedDocument = await this.resolver.resolve(didDocument.id);
                didRecord = await didRepository.findCreatedDid(agentContext, didDocument.id);
                if (!resolvedDocument.didDocument || resolvedDocument.didDocumentMetadata.deactivated || !didRecord) {
                    return {
                        didDocumentMetadata: {},
                        didRegistrationMetadata: {},
                        didState: {
                            state: 'failed',
                            reason: 'Did not found',
                        },
                    };
                }
                if ((_a = options === null || options === void 0 ? void 0 : options.secret) === null || _a === void 0 ? void 0 : _a.privateKey) {
                    const privateKey = (_b = options === null || options === void 0 ? void 0 : options.secret) === null || _b === void 0 ? void 0 : _b.privateKey;
                    if (privateKey && !(0, core_1.isValidPrivateKey)(privateKey, core_1.KeyType.K256)) {
                        return {
                            didDocumentMetadata: {},
                            didRegistrationMetadata: {},
                            didState: {
                                state: 'failed',
                                reason: 'Invalid private key provided',
                            },
                        };
                    }
                    const key = await agentContext.wallet.createKey({
                        keyType: core_1.KeyType.K256,
                        privateKey: privateKey,
                    });
                    const verificationMethodCount = (_d = (_c = didDocument === null || didDocument === void 0 ? void 0 : didDocument.verificationMethod) === null || _c === void 0 ? void 0 : _c.length) !== null && _d !== void 0 ? _d : 0;
                    const verificationMethod = (0, core_1.getEcdsaSecp256k1VerificationKey2019)({
                        id: `${didDocument.id}#key-${verificationMethodCount + 1}`,
                        key,
                        controller: didDocument.id,
                    });
                    didDocument.verificationMethod = [...((_e = didDocument === null || didDocument === void 0 ? void 0 : didDocument.verificationMethod) !== null && _e !== void 0 ? _e : []), verificationMethod];
                }
            }
            else {
                return {
                    didDocumentMetadata: {},
                    didRegistrationMetadata: {},
                    didState: {
                        state: 'failed',
                        reason: isValidDidDoc !== null && isValidDidDoc !== void 0 ? isValidDidDoc : 'Provide a valid didDocument',
                    },
                };
            }
            if (!didRecord) {
                return {
                    didDocumentMetadata: {},
                    didRegistrationMetadata: {},
                    didState: {
                        state: 'failed',
                        reason: 'DidRecord not found in wallet',
                    },
                };
            }
            const publicKeyBase58 = await this.getPublicKeyFromDid(agentContext, options.did);
            if (!publicKeyBase58) {
                throw new core_1.CredoError('Public Key not found in wallet');
            }
            const signingKey = await this.getSigningKey(agentContext.wallet, publicKeyBase58);
            const didRegistry = ledgerService.createDidRegistryInstance(signingKey);
            const response = await didRegistry.update(didDocument.id, didDocument);
            if (!response) {
                throw new Error('Unable to update did document');
            }
            // Save the did document
            didRecord.didDocument = didDocument;
            await didRepository.update(agentContext, didRecord);
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {
                    txn: response.txnHash,
                },
                didState: {
                    state: 'finished',
                    did: didDocument.id,
                    didDocument,
                },
            };
        }
        catch (error) {
            const errorMessage = error instanceof Error ? error.message : 'Unknown error';
            agentContext.config.logger.error(`Error Updating DID : ${errorMessage}`);
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: `unknownError: ${errorMessage}`,
                },
            };
        }
    }
    async deactivate(agentContext, options) {
        const didRepository = agentContext.dependencyManager.resolve(core_1.DidRepository);
        const ledgerService = agentContext.dependencyManager.resolve(ledger_1.PolygonLedgerService);
        const did = options.did;
        try {
            const { didDocument, didDocumentMetadata } = await this.resolver.resolve(did);
            const didRecord = await didRepository.findCreatedDid(agentContext, did);
            if (!didDocument || didDocumentMetadata.deactivated || !didRecord) {
                return {
                    didDocumentMetadata: {},
                    didRegistrationMetadata: {},
                    didState: {
                        state: 'failed',
                        reason: 'Did not found',
                    },
                };
            }
            const publicKeyBase58 = await this.getPublicKeyFromDid(agentContext, options.did);
            if (!publicKeyBase58) {
                throw new core_1.CredoError('Public Key not found in wallet');
            }
            const signingKey = await this.getSigningKey(agentContext.wallet, publicKeyBase58);
            const didRegistry = ledgerService.createDidRegistryInstance(signingKey);
            const updatedDidDocument = new core_1.DidDocumentBuilder(options.did).addContext('https://www.w3.org/ns/did/v1').build();
            const response = await didRegistry.update(didDocument.id, updatedDidDocument);
            if (!response) {
                throw new core_1.CredoError(`Unable to deactivate did document for did : ${did}`);
            }
            await didRepository.update(agentContext, didRecord);
            return {
                didDocumentMetadata: {
                    deactivated: true,
                },
                didRegistrationMetadata: {
                    txn: response.txnHash,
                },
                didState: {
                    state: 'finished',
                    did: didDocument.id,
                    didDocument: core_1.JsonTransformer.fromJSON(didDocument, core_1.DidDocument),
                    secret: options.secret,
                },
            };
        }
        catch (error) {
            const errorMessage = error instanceof Error ? error.message : 'Unknown error';
            agentContext.config.logger.error(`Error deactivating DID ${errorMessage}`);
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: `unknownError: ${errorMessage}`,
                },
            };
        }
    }
    async getSigningKey(wallet, publicKeyBase58) {
        if (!(wallet instanceof askar_1.AskarWallet) && !(wallet instanceof askar_1.AskarProfileWallet)) {
            throw new core_1.CredoError('Incorrect wallet type: Polygon Module currently only supports Askar wallet');
        }
        const keyEntry = await wallet.withSession(async (session) => await session.fetchKey({ name: publicKeyBase58 }));
        if (!keyEntry) {
            throw new core_1.WalletError('Key not found in wallet');
        }
        const signingKey = new ethers_1.SigningKey(keyEntry.key.secretBytes);
        keyEntry.key.handle.free();
        return signingKey;
    }
    async getPublicKeyFromDid(agentContext, did) {
        var _a;
        const didRepository = agentContext.dependencyManager.resolve(core_1.DidRepository);
        const didRecord = await didRepository.findCreatedDid(agentContext, did);
        if (!didRecord) {
            throw new core_1.CredoError('DidRecord not found');
        }
        if (!((_a = didRecord.didDocument) === null || _a === void 0 ? void 0 : _a.verificationMethod)) {
            throw new core_1.CredoError('VerificationMethod not found cannot get public key');
        }
        const publicKeyBase58 = didRecord.didDocument.verificationMethod[0].publicKeyBase58;
        return publicKeyBase58;
    }
}
exports.PolygonDidRegistrar = PolygonDidRegistrar;
//# sourceMappingURL=PolygonDidRegistrar.js.map